@@ -17,6 +17,7 @@ KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
 """
+from six import string_types, PY2
 import mimetypes
 from riak.util import lazy_property
 
@@ -50,13 +51,14 @@ class RiakBucket(object):
         :param bucket_type: The parent bucket type of this bucket
         :type bucket_type: :class:`BucketType`
         """
-        try:
-            if isinstance(name, basestring):
-                name = name.encode('ascii')
-            else:
-                raise TypeError('Bucket name must be a string')
-        except UnicodeError:
-            raise TypeError('Unicode bucket names are not supported.')
+        if PY2:
+            try:
+                if isinstance(name, string_types):
+                        name = name.encode('ascii')
+                else:
+                    raise TypeError('Bucket name must be a string')
+            except UnicodeError:
+                raise TypeError('Unicode bucket names are not supported.')
 
         if not isinstance(bucket_type, BucketType):
             raise TypeError('Parent bucket type must be a BucketType instance')
@@ -173,11 +175,12 @@ class RiakBucket(object):
         if self.bucket_type.datatype:
             return TYPES[self.bucket_type.datatype](bucket=self, key=key)
 
-        try:
-            if isinstance(data, basestring):
-                data = data.encode('ascii')
-        except UnicodeError:
-            raise TypeError('Unicode data values are not supported.')
+        if PY2:
+            try:
+                if isinstance(data, string_types):
+                    data = data.encode('ascii')
+            except UnicodeError:
+                raise TypeError('Unicode data values are not supported.')
 
         obj = RiakObject(self._client, self, key)
         obj.content_type = content_type
@@ -411,7 +414,12 @@ class RiakBucket(object):
             binary_data = bytearray(binary_data)
         if not mimetype:
             mimetype = 'application/octet-stream'
-        return self.new(key, encoded_data=binary_data, content_type=mimetype)
+        if PY2:
+            return self.new(key, encoded_data=binary_data,
+                            content_type=mimetype)
+        else:
+            return self.new(key, encoded_data=bytes(binary_data),
+                            content_type=mimetype)
 
     def search_enabled(self):
         """
@@ -730,5 +738,5 @@ class BucketType(object):
             return True
 
 
-from riak_object import RiakObject
+from riak.riak_object import RiakObject
 from riak.datatypes import TYPES
