@@ -16,10 +16,15 @@ specific language governing permissions and limitations
 under the License.
 """
 
+from __future__ import print_function
 from collections import namedtuple
-from Queue import Queue
 from threading import Thread, Lock, Event
 from multiprocessing import cpu_count
+from six import PY2
+if PY2:
+    from Queue import Queue
+else:
+    from queue import Queue
 
 __all__ = ['multiget', 'MultiGetPool']
 
@@ -202,15 +207,15 @@ if __name__ == '__main__':
     from riak import RiakClient
     import riak.benchmark as benchmark
     client = RiakClient(protocol='pbc')
-    bkeys = [('default', 'multiget', str(key)) for key in xrange(10000)]
+    bkeys = [('default', 'multiget', str(key)) for key in range(10000)]
 
     data = open(__file__).read()
 
-    print "Benchmarking multiget:"
-    print "      CPUs: {0}".format(cpu_count())
-    print "   Threads: {0}".format(POOL_SIZE)
-    print "      Keys: {0}".format(len(bkeys))
-    print
+    print("Benchmarking multiget:")
+    print("      CPUs: {0}".format(cpu_count()))
+    print("   Threads: {0}".format(POOL_SIZE))
+    print("      Keys: {0}".format(len(bkeys)))
+    print()
 
     with benchmark.measure() as b:
         with b.report('populate'):
