@@ -1,5 +1,7 @@
 # -*- coding: utf-8 -*-
 
+from __future__ import print_function
+from six import PY2
 from riak.mapreduce import RiakMapReduce
 from riak import key_filter, RiakError
 from riak.tests.test_yokozuna import wait_for_yz_index
@@ -17,12 +19,20 @@ class LinkTests(object):
     def test_store_and_get_links(self):
         # Create the object...
         bucket = self.client.bucket(self.bucket_name)
-        bucket.new(key=self.key_name, encoded_data='2',
-                   content_type='application/octet-stream') \
-            .add_link(bucket.new("foo1")) \
-            .add_link(bucket.new("foo2"), "tag") \
-            .add_link(bucket.new("foo3"), "tag2!@#%^&*)") \
-            .store()
+        if PY2:
+            bucket.new(key=self.key_name, encoded_data='2',
+                       content_type='application/octet-stream') \
+                .add_link(bucket.new("foo1")) \
+                .add_link(bucket.new("foo2"), "tag") \
+                .add_link(bucket.new("foo3"), "tag2!@#%^&*)") \
+                .store()
+        else:
+            bucket.new(key=self.key_name, data='2',
+                       content_type='application/octet-stream') \
+                .add_link(bucket.new("foo1")) \
+                .add_link(bucket.new("foo2"), "tag") \
+                .add_link(bucket.new("foo3"), "tag2!@#%^&*)") \
+                .store()
         obj = bucket.get(self.key_name)
         links = obj.links
         self.assertEqual(len(links), 3)
@@ -88,6 +98,23 @@ class ErlangMapReduceTests(object):
             .run()
         self.assertEqual(len(result), 2)
 
+    def test_erlang_map_reduce_bucket_type(self):
+        # Create the object...
+        btype = self.client.bucket_type("pytest")
+        bucket = btype.bucket(self.bucket_name)
+        bucket.new("foo", 2).store()
+        bucket.new("bar", 2).store()
+        bucket.new("baz", 4).store()
+        # Run the map...
+        result = self.client \
+            .add(self.bucket_name, "foo", bucket_type="pytest") \
+            .add(self.bucket_name, "bar", bucket_type="pytest") \
+            .add(self.bucket_name, "baz", bucket_type="pytest") \
+            .map(["riak_kv_mapreduce", "map_object_value"]) \
+            .reduce(["riak_kv_mapreduce", "reduce_set_union"]) \
+            .run()
+        self.assertEqual(len(result), 2)
+
     def test_erlang_source_map_reduce(self):
         # Create the object...
         bucket = self.client.bucket(self.bucket_name)
@@ -95,6 +122,7 @@ class ErlangMapReduceTests(object):
         bucket.new("bar", 3).store()
         bucket.new("baz", 4).store()
         strfun_allowed = True
+        result = []
         # Run the map...
         try:
             result = self.client \
@@ -105,6 +133,32 @@ class ErlangMapReduceTests(object):
             Value = riak_object:get_value(Object),
             [Value]
         end.""", {'language': 'erlang'}).run()
+        except RiakError as e:
+            if e.value.startswith('May have tried'):
+                strfun_allowed = False
+            else:
+                print("test_erlang_source_map_reduce {}".format(e.value))
+        if strfun_allowed:
+            self.assertEqual(result, ['2', '3', '4'])
+
+    def test_erlang_source_map_reduce_bucket_type(self):
+        # Create the object...
+        btype = self.client.bucket_type("pytest")
+        bucket = btype.bucket(self.bucket_name)
+        bucket.new("foo", 2).store()
+        bucket.new("bar", 3).store()
+        bucket.new("baz", 4).store()
+        strfun_allowed = True
+        # Run the map...
+        try:
+            result = self.client \
+                .add(self.bucket_name, "foo", bucket_type="pytest") \
+                .add(self.bucket_name, "bar", bucket_type="pytest") \
+                .add(self.bucket_name, "baz", bucket_type="pytest") \
+                .map("""fun(Object, _KD, _A) ->
+            Value = riak_object:get_value(Object),
+            [Value]
+        end.""", {'language': 'erlang'}).run()
         except RiakError as e:
             if e.value.startswith('May have tried'):
                 strfun_allowed = False
@@ -147,20 +201,29 @@ class JSMapReduceTests(object):
         # test ASCII-encodable unicode is accepted
         mr.map(u"function (v) { return [JSON.parse(v.values[0].data)]; }")
 
-        # test non-ASCII-encodable unicode is rejected
-        self.assertRaises(TypeError, mr.map,
-                          u"""
-                          function (v) {
-                          /* æ */
-                            return [JSON.parse(v.values[0].data)];
-                          }""")
+        # test non-ASCII-encodable unicode is rejected in Python 2.x
+        if PY2:
+            self.assertRaises(TypeError, mr.map,
+                              u"""
+                              function (v) {
+                              /* æ */
+                                return [JSON.parse(v.values[0].data)];
+                              }""")
+        else:
+            mr = self.client.add(self.bucket_name, "foo")
+            result = mr.map("""function (v) {
+                      /* æ */
+                        return [JSON.parse(v.values[0].data)];
+                      }""").run()
+            self.assertEqual(result, [2])
 
-        # test non-ASCII-encodable string is rejected
-        self.assertRaises(TypeError, mr.map,
-                          """function (v) {
-                               /* æ */
-                               return [JSON.parse(v.values[0].data)];
-                             }""")
+        # test non-ASCII-encodable string is rejected in Python 2.x
+        if PY2:
+            self.assertRaises(TypeError, mr.map,
+                              """function (v) {
+                                   /* æ */
+                                   return [JSON.parse(v.values[0].data)];
+                                 }""")
 
     def test_javascript_named_map(self):
         # Create the object...
@@ -173,6 +236,18 @@ class JSMapReduceTests(object):
             .run()
         self.assertEqual(result, [2])
 
+    def test_javascript_named_map_bucket_type(self):
+        # Create the object...
+        btype = self.client.bucket_type("pytest")
+        bucket = btype.bucket(self.bucket_name)
+        bucket.new("foo", 2).store()
+        # Run the map...
+        result = self.client \
+            .add(self.bucket_name, "foo", bucket_type="pytest") \
+            .map("Riak.mapValuesJson") \
+            .run()
+        self.assertEqual(result, [2])
+
     def test_javascript_source_map_reduce(self):
         # Create the object...
         bucket = self.client.bucket(self.bucket_name)
@@ -189,6 +264,23 @@ class JSMapReduceTests(object):
             .run()
         self.assertEqual(result, [3])
 
+    def test_javascript_source_map_reduce_bucket_type(self):
+        # Create the object...
+        btype = self.client.bucket_type("pytest")
+        bucket = btype.bucket(self.bucket_name)
+        bucket.new("foo", 2).store()
+        bucket.new("bar", 3).store()
+        bucket.new("baz", 4).store()
+        # Run the map...
+        result = self.client \
+            .add(self.bucket_name, "foo", bucket_type="pytest") \
+            .add(self.bucket_name, "bar", bucket_type="pytest") \
+            .add(self.bucket_name, "baz", bucket_type="pytest") \
+            .map("function (v) { return [1]; }") \
+            .reduce("Riak.reduceSum") \
+            .run()
+        self.assertEqual(result, [3])
+
     def test_javascript_named_map_reduce(self):
         # Create the object...
         bucket = self.client.bucket(self.bucket_name)
@@ -205,6 +297,23 @@ class JSMapReduceTests(object):
             .run()
         self.assertEqual(result, [9])
 
+    def test_javascript_named_map_reduce_bucket_type(self):
+        # Create the object...
+        btype = self.client.bucket_type("pytest")
+        bucket = btype.bucket(self.bucket_name)
+        bucket.new("foo", 2).store()
+        bucket.new("bar", 3).store()
+        bucket.new("baz", 4).store()
+        # Run the map...
+        result = self.client \
+            .add(self.bucket_name, "foo", bucket_type="pytest") \
+            .add(self.bucket_name, "bar", bucket_type="pytest") \
+            .add(self.bucket_name, "baz", bucket_type="pytest") \
+            .map("Riak.mapValuesJson") \
+            .reduce("Riak.reduceSum") \
+            .run()
+        self.assertEqual(result, [9])
+
     def test_javascript_bucket_map_reduce(self):
         # Create the object...
         bucket = self.client.bucket("bucket_%s" % self.randint())
@@ -219,6 +328,21 @@ class JSMapReduceTests(object):
             .run()
         self.assertEqual(result, [9])
 
+    def test_javascript_bucket_map_reduceP_bucket_type(self):
+        # Create the object...
+        btype = self.client.bucket_type("pytest")
+        bucket = btype.bucket("bucket_%s" % self.randint())
+        bucket.new("foo", 2).store()
+        bucket.new("bar", 3).store()
+        bucket.new("baz", 4).store()
+        # Run the map...
+        result = self.client \
+            .add(bucket.name, bucket_type="pytest") \
+            .map("Riak.mapValuesJson") \
+            .reduce("Riak.reduceSum") \
+            .run()
+        self.assertEqual(result, [9])
+
     def test_javascript_arg_map_reduce(self):
         # Create the object...
         bucket = self.client.bucket(self.bucket_name)
@@ -235,6 +359,23 @@ class JSMapReduceTests(object):
             .run()
         self.assertEqual(result, [10])
 
+    def test_javascript_arg_map_reduce_bucket_type(self):
+        # Create the object...
+        btype = self.client.bucket_type("pytest")
+        bucket = btype.bucket(self.bucket_name)
+        bucket.new("foo", 2).store()
+        # Run the map...
+        result = self.client \
+            .add(self.bucket_name, "foo", 5, bucket_type="pytest") \
+            .add(self.bucket_name, "foo", 10, bucket_type="pytest") \
+            .add(self.bucket_name, "foo", 15, bucket_type="pytest") \
+            .add(self.bucket_name, "foo", -15, bucket_type="pytest") \
+            .add(self.bucket_name, "foo", -5, bucket_type="pytest") \
+            .map("function(v, arg) { return [arg]; }") \
+            .reduce("Riak.reduceSum") \
+            .run()
+        self.assertEqual(result, [10])
+
     def test_key_filters(self):
         bucket = self.client.bucket("kftest")
         bucket.new("basho-20101215", 1).store()
@@ -250,6 +391,22 @@ class JSMapReduceTests(object):
 
         self.assertEqual(result, ["yahoo-20090613"])
 
+    def test_key_filters_bucket_type(self):
+        btype = self.client.bucket_type("pytest")
+        bucket = btype.bucket("kftest")
+        bucket.new("basho-20101215", 1).store()
+        bucket.new("google-20110103", 2).store()
+        bucket.new("yahoo-20090613", 3).store()
+
+        result = self.client \
+            .add("kftest", bucket_type="pytest") \
+            .add_key_filters([["tokenize", "-", 2]]) \
+            .add_key_filter("ends_with", "0613") \
+            .map("function (v, keydata) { return [v.key]; }") \
+            .run()
+
+        self.assertEqual(result, ["yahoo-20090613"])
+
     def test_key_filters_f_chain(self):
         bucket = self.client.bucket("kftest")
         bucket.new("basho-20101215", 1).store()
@@ -387,7 +544,7 @@ class JSMapReduceTests(object):
             return [solr_doc["calories_i"]]; }""")
         result = mr.reduce('function(values, arg) ' +
                            '{ return [values.sort()[0]]; }').run()
-        self.assertEquals(result, [100])
+        self.assertEqual(result, [100])
 
 
 class MapReduceAliasTests(object):
@@ -396,10 +553,16 @@ class MapReduceAliasTests(object):
     def test_map_values(self):
         # Add a value to the bucket
         bucket = self.client.bucket(self.bucket_name)
-        bucket.new('one', encoded_data='value_1',
-                   content_type='text/plain').store()
-        bucket.new('two', encoded_data='value_2',
-                   content_type='text/plain').store()
+        if PY2:
+            bucket.new('one', encoded_data='value_1',
+                       content_type='text/plain').store()
+            bucket.new('two', encoded_data='value_2',
+                       content_type='text/plain').store()
+        else:
+            bucket.new('one', data='value_1',
+                       content_type='text/plain').store()
+            bucket.new('two', data='value_2',
+                       content_type='text/plain').store()
 
         # Create a map reduce object and use one and two as inputs
         mr = self.client.add(self.bucket_name, 'one')\
@@ -610,4 +773,7 @@ class MapReduceStreamTests(object):
 
         # This should not raise an exception
         obj = bucket.get('one')
-        self.assertEqual('1', obj.encoded_data)
+        if PY2:
+            self.assertEqual('1', obj.encoded_data)
+        else:
+            self.assertEqual(b'1', obj.encoded_data)
