@@ -20,10 +20,11 @@ under the License.
 import base64
 import random
 import threading
-import platform
 import os
 import json
-from feature_detect import FeatureDetection
+import platform
+from six import PY2
+from riak.transports.feature_detect import FeatureDetection
 
 
 class RiakTransport(FeatureDetection):
@@ -46,8 +47,13 @@ class RiakTransport(FeatureDetection):
         """
         Returns a random client identifier
         """
-        return ('py_%s' %
-                base64.b64encode(str(random.randint(1, 0x40000000))))
+        if PY2:
+            return ('py_%s' %
+                    base64.b64encode(str(random.randint(1, 0x40000000))))
+        else:
+            return ('py_%s' %
+                    base64.b64encode(bytes(str(random.randint(1, 0x40000000)),
+                                     'ascii')))
 
     @classmethod
     def make_fixed_client_id(self):
@@ -65,7 +71,8 @@ class RiakTransport(FeatureDetection):
         """
         raise NotImplementedError
 
-    def get(self, robj, r=None, pr=None, timeout=None):
+    def get(self, robj, r=None, pr=None, timeout=None, basic_quorum=None,
+            notfound_ok=None):
         """
         Fetches an object.
         """
@@ -85,13 +92,13 @@ class RiakTransport(FeatureDetection):
         """
         raise NotImplementedError
 
-    def get_buckets(self, timeout=None):
+    def get_buckets(self, bucket_type=None, timeout=None):
         """
         Gets the list of buckets as strings.
         """
         raise NotImplementedError
 
-    def stream_buckets(self, timeout=None):
+    def stream_buckets(self, bucket_type=None, timeout=None):
         """
         Streams the list of buckets through an iterator
         """
@@ -109,6 +116,18 @@ class RiakTransport(FeatureDetection):
         """
         raise NotImplementedError
 
+    def get_bucket_type_props(self, bucket_type):
+        """
+        Fetches properties for the given bucket-type.
+        """
+        raise NotImplementedError
+
+    def set_bucket_type_props(self, bucket_type, props):
+        """
+        Sets properties on the given bucket-type.
+        """
+        raise NotImplementedError
+
     def clear_bucket_props(self, bucket):
         """
         Reset bucket properties to their defaults
@@ -153,6 +172,42 @@ class RiakTransport(FeatureDetection):
         """
         raise NotImplementedError
 
+    def create_search_index(self, index, schema=None, n_val=None):
+        """
+        Creates a yokozuna search index.
+        """
+        raise NotImplementedError
+
+    def get_search_index(self, index):
+        """
+        Returns a yokozuna search index or None.
+        """
+        raise NotImplementedError
+
+    def list_search_indexes(self):
+        """
+        Lists all yokozuna search indexes.
+        """
+        raise NotImplementedError
+
+    def delete_search_index(self, index):
+        """
+        Deletes a yokozuna search index.
+        """
+        raise NotImplementedError
+
+    def create_search_schema(self, schema, content):
+        """
+        Creates a yokozuna search schema.
+        """
+        raise NotImplementedError
+
+    def get_search_schema(self, schema):
+        """
+        Returns a yokozuna search schema.
+        """
+        raise NotImplementedError
+
     def search(self, index, query, **params):
         """
         Performs a search query.
@@ -201,6 +256,20 @@ class RiakTransport(FeatureDetection):
         """
         raise NotImplementedError
 
+    def fetch_datatype(self, bucket, key, r=None, pr=None, basic_quorum=None,
+                       notfound_ok=None, timeout=None, include_context=None):
+        """
+        Fetches a Riak Datatype.
+        """
+        raise NotImplementedError
+
+    def update_datatype(self, datatype, w=None, dw=None, pw=None,
+                        return_body=None, timeout=None, include_context=None):
+        """
+        Updates a Riak Datatype by sending local operations to the server.
+        """
+        raise NotImplementedError
+
     def _search_mapred_emu(self, index, query):
         """
         Emulates a search request via MapReduce. Used in the case
@@ -262,3 +331,10 @@ class RiakTransport(FeatureDetection):
 
         content = json.dumps(job)
         return content
+
+    def _check_bucket_types(self, bucket_type):
+        if not self.bucket_types():
+            raise NotImplementedError('Server does not support bucket-types')
+
+        if bucket_type.is_default():
+            raise ValueError('Cannot manipulate the default bucket-type')
